// dojstreams - a c++ stream library implementation
//
// Copyright (c) 2003-2004 by Dirk Jagdmann <doj@cubic.org>
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any
// damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any
// purpose, including commercial applications, and to alter it and
// redistribute it freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
// not claim that you wrote the original software. If you use this
// software in a product, an acknowledgment in the product
// documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must
// not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source
// distribution.

#ifndef BASICFSTREAM__HPP
#define BASICFSTREAM__HPP

#include <ds_iostream.hpp>
#include <ds_basicfilebuf.hpp>

namespace DS_NAMESPACE
{

  template <TYPENAME charT>
  class basic_fstream : public basic_iostream<charT>
  {
    typedef basic_ios<charT> bios;
  private:
    basic_filebuf<charT> sb;

  public:
    basic_fstream()
    {
      bios::rdbuf(&sb);
    }

    explicit basic_fstream(const char *filename, ios_base::openmode mode = ios_base::in | ios_base::out)
    {
      bios::rdbuf(&sb);
      open(filename, mode);
    }

    basic_filebuf<charT> *rdbuf()
    {
      return &sb;
    }

    bool is_open() const
    {
      return this->rdbuf()->is_open();
    }

    void open(const char *filename, ios_base::openmode mode = ios_base::in | ios_base::out)
    {
      this->setstate(sb.open(filename, mode)?ios::goodbit:ios::failbit);
    }

    void close()
    {
      this->rdbuf()->close();
    }
  };

  typedef basic_fstream<char> fstream;
  typedef basic_fstream<wchar_t> wfstream;

}

#endif
