// dojstreams - a c++ stream library implementation
//
// Copyright (c) 2003-2004 by Dirk Jagdmann <doj@cubic.org>
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any
// damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any
// purpose, including commercial applications, and to alter it and
// redistribute it freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
// not claim that you wrote the original software. If you use this
// software in a product, an acknowledgment in the product
// documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must
// not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source
// distribution.

#ifndef DSBASICOSTRINGSTREAM__HPP
#define DSBASICOSTRINGSTREAM__HPP

#include <ds_basicstringbuf.hpp>
#include <ds_basicostream.hpp>

namespace DS_NAMESPACE
{
  template <TYPENAME charT>
  class basic_ostringstream : public basic_ostream<charT>
  {
    typedef basic_ios<charT> bios;
    basic_stringbuf<charT> buf;
  public:
    explicit basic_ostringstream(ios_base::openmode mode = ios_base::out) :
      buf(mode)
    {
      bios::rdbuf(&buf);
    }

    explicit basic_ostringstream(const DS_STRING_NAMESPACE::basic_string<charT>& str, ios_base::openmode mode = ios_base::out) :
      buf(str, mode)
    {
      bios::rdbuf(&buf);
    }

    basic_stringbuf<charT> *rdbuf() { return &buf; }
    DS_STRING_NAMESPACE::basic_string<charT> str() { return buf.str(); }
    void str(const DS_STRING_NAMESPACE::basic_string<charT>& newstr) { buf.str(newstr); }
    };

  typedef basic_ostringstream<char>    ostringstream;
  typedef basic_ostringstream<wchar_t> wostringstream;
}

#endif
