// dojstreams - a c++ stream library implementation
//
// Copyright (c) 2003-2004 by Dirk Jagdmann <doj@cubic.org>
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any
// damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any
// purpose, including commercial applications, and to alter it and
// redistribute it freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
// not claim that you wrote the original software. If you use this
// software in a product, an acknowledgment in the product
// documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must
// not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source
// distribution.

#include <ds_basicistream.hpp>
#include <ds_basicostream.hpp>
#include <ds_basicfilebuf.hpp>

#include <stdio.h>
#include <string.h>

namespace DS_NAMESPACE
{
  extern istream cin;
  extern ostream cout;
  extern ostream cerr;
  extern ostream clog;
  extern wistream wcin;
  extern wostream wcout;
  extern wostream wcerr;
  extern wostream wclog;

  static filebuf *insb, *outsb, *errsb;
  static wfilebuf *winsb, *woutsb, *werrsb;
  static int ds_debughook_canflush;

  int ios_base::Init::count;
  ios_base::Init::Init()
  {
    if(++count == 1)
      {
	insb=new filebuf;
	insb->open(stdin);
	outsb=new filebuf;
	outsb->open(stdout);
	errsb=new filebuf;
	errsb->open(stderr);

	winsb=new wfilebuf;
	winsb->open(stdin);
	woutsb=new wfilebuf;
	woutsb->open(stdout);
	werrsb=new wfilebuf;
	werrsb->open(stderr);

#if defined(_WIN32_WCE)
	cin.rdbuf(insb);
	cout.rdbuf(outsb);
	cerr.rdbuf(errsb);
	clog.rdbuf(errsb);
#else
	new (&cin) istream(insb);
	new (&cout) ostream(outsb);
	new (&cerr) ostream(errsb);
	new (&clog) ostream(errsb);
#endif
	cin.tie(&cout);
	cerr.flags(ios_base::unitbuf);

#if defined(_WIN32_WCE)
	wcin.rdbuf(winsb);
	wcout.rdbuf(woutsb);
	wcerr.rdbuf(werrsb);
	wclog.rdbuf(werrsb);
#else
	new (&wcin) wistream(winsb);
	new (&wcout) wostream(woutsb);
	new (&wcerr) wostream(werrsb);
	new (&wclog) wostream(werrsb);
#endif
	wcin.tie(&wcout);
	wcerr.flags(ios_base::unitbuf);

	ds_debughook_canflush=1;
      }
  }

  ios_base::Init::~Init()
  {
    if(--count == 0)
      {
	wclog.~wostream();
	wcerr.~wostream();
	wcout.~wostream();
	wcin.~wistream();

	clog.~ostream();
	cerr.~ostream();
	cout.~ostream();
	cin.~istream();

	delete insb;
	delete outsb;
	delete errsb;
	delete winsb;
	delete woutsb;
	delete werrsb;
      }
  }

  ////// ios_base

  bool ios_base::sync_with_stdio(bool sync)
  {
    // todo
    return true;
  }

  int ios_base::xalloc()
  {
    static int i=0;
    return ++i;
  }

  ios_base::~ios_base()
  {
    do_callbacks(erase_event);
  }

#if !defined(_WIN32_WCE)
  void ios_base::register_callback(ios_base::event_callback pfn, int idx)
  {
#if 0
    ecb_vec.push_back(pfn);
    idx_vec.push_back(idx);
#endif
  }
#endif

  void ios_base::do_callbacks(event e)
  {
#if 0
    for(unsigned i=0; i<idx_vec.size(); ++i)
      ecb_vec[i](e, idx_vec[i]);
#endif
  }

  void ios_base::copyfmt(const ios_base& i)
  {
    do_callbacks(erase_event);
    width_=i.width_;
    precision_=i.precision_;
    fmtflags_=i.fmtflags_;
    do_callbacks(copyfmt_event);
    // copy exception mask
  }

  void*& ios_base::pword(int idx)
  {
    if(idx<0)
      return void_vec[0];
    void_vec.resize(idx);
    if(void_vec.size()>=(unsigned)idx)
      return void_vec[idx];
    return void_vec[0];
  }

  long&  ios_base::iword(int idx)
  {
    if(idx<0)
      return long_vec[0];
    long_vec.resize(idx);
    if(long_vec.size()>=(unsigned)idx)
      return long_vec[idx];
    return long_vec[0];
  }

  /////////////

  char *truestr="true";
  char *falsestr="false";
  char *zerostr="0";
  wchar_t *truewstr=L"true";
  wchar_t *falsewstr=L"false";
  wchar_t *zerowstr=L"0";

  char *uppernumbers=     "0123456789ABCDEFX";
  char *lowernumbers=     "0123456789abcdefx";
  wchar_t *upperwnumbers=L"0123456789ABCDEFX";
  wchar_t *lowerwnumbers=L"0123456789abcdefx";

#if DSDEBUG
  void ds_debughook(const char* str, const int i)
  {
    static DS_STRING_NAMESPACE::string toll;
    if(str[0])
      {
	toll+="ds_debughook():";
	char buf[1024];
	snprintf(buf, sizeof(buf), str, i);
	toll+=buf;
	toll+='\n';
      }
    if(ds_debughook_canflush)
      {
	write(2, toll.data(), toll.size());
	fsync(2);
	toll.erase();
      }
  }
#endif

}
