// dojstreams - a c++ stream library implementation
//
// Copyright (c) 2003-2004 by Dirk Jagdmann <doj@cubic.org>
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any
// damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any
// purpose, including commercial applications, and to alter it and
// redistribute it freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
// not claim that you wrote the original software. If you use this
// software in a product, an acknowledgment in the product
// documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must
// not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source
// distribution.

#ifndef DS_STREAMITERATOR__HPP
#define DS_STREAMITERATOR__HPP

#include <ds_basicostream.hpp>
#include <ds_basicistream.hpp>

namespace DS_NAMESPACE
{
  template <TYPENAME T, TYPENAME charT = char>
  class istream_iterator
  {
  protected:
    basic_istream<charT> *is;
    bool ok;
    T value;

    void read()
    {
      if(is)
	{
	  (*is) >> value;
	  ok=is->good();
	}
    }

  public:
    istream_iterator<T,charT>()
      : is(0), ok(false)
    { }

    istream_iterator<T,charT>(basic_istream<charT>& s)
      : is(&s), ok(s.good())
    { }

    bool operator== (const istream_iterator<T,charT>& s) const
    {
      if(is && s.is)
	return is==s.is && ok==s.ok;
      return ok==s.ok;
    }

    bool operator!= (const istream_iterator<T,charT>& s) const
    {
      return !(*this==s);
    }
    const T& operator* () const { return value; }
    const T* operator->() const { return &value; }

    istream_iterator<T,charT>& operator++ ()
    {
      read();
      return *this;
    }

    istream_iterator<T,charT>& operator++ (int)
    {
      read();
      return *this;
    }
  };

  template <TYPENAME T, TYPENAME charT = char>
  class ostream_iterator
  {
  protected:
    basic_ostream<charT> *os;
    charT delim;
  public:
    ostream_iterator<T,charT>()
      : os(0)
    { }

    ostream_iterator<T,charT>(basic_ostream<charT>& s, charT d=' ')
      : os(&s), delim(d)
    { }

    ostream_iterator<T,charT>& operator= (const T& t)
    {
      if(os)
	{
	  (*os) << t;
	  (*os) << delim;
	}
      return *this;
    }

    ostream_iterator<T,charT>& operator++ ()    { return *this; }
    ostream_iterator<T,charT>& operator++ (int) { return *this; }
    ostream_iterator<T,charT>& operator*  ()    { return *this; }
  };

}

#endif
