// dojstreams - a c++ stream library implementation
//
// Copyright (c) 2003-2004 by Dirk Jagdmann <doj@cubic.org>
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any
// damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any
// purpose, including commercial applications, and to alter it and
// redistribute it freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
// not claim that you wrote the original software. If you use this
// software in a product, an acknowledgment in the product
// documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must
// not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source
// distribution.

#include "inc.hpp"
#include <math.h>
#include <string.h>
#include <assert.h>

DS_STRING_NAMESPACE::string suffix;

void fullrange()
{
  ios::fmtflags tab[] = {
    ios::left,
    ios::right,
    ios::internal,
    static_cast<ios::fmtflags>(ios::left|ios::right),
    static_cast<ios::fmtflags>(ios::left|ios::internal),
    static_cast<ios::fmtflags>(ios::right|ios::internal),
    static_cast<ios::fmtflags>(ios::left|ios::right|ios::internal),
    static_cast<ios::fmtflags>(0)
  };

  static int tabi=0;

  const unsigned int add=82734223;

  cout << "unsigned" << endl;
  unsigned int i=0;
  cout << i << endl;
  i+=add+1;
  while(i>add)
    {
      cout.setf(tab[tabi++], ios::adjustfield);
      tabi%=sizeof(tab)/sizeof(tab[0]);

      cout << setw(-10) << i << endl;
      i+=add;
    }

  cout << "signed" << endl;
  int j=0;
  cout << j << endl;
  j+=add+1;
  cout << setfill('$');
  while(j>(int)add || j<0)
    {
      cout.setf(tab[tabi++], ios::adjustfield);
      tabi%=sizeof(tab)/sizeof(tab[0]);

      cout << setw(40) << j << endl;
      j+=add;
    }
  cout << setfill(' ');
}

void fullbase()
{
  ios::fmtflags tab[] = {
    ios::dec,
    ios::oct,
    ios::hex,
    (ios::fmtflags)(ios::dec | ios::oct),
    (ios::fmtflags)(ios::dec | ios::hex),
    (ios::fmtflags)(ios::oct | ios::hex),
    (ios::fmtflags)(ios::dec | ios::oct | ios::hex),
  };

  char* str[] = {
    "dec",
    "oct",
    "hex",
    "dec|oct",
    "dec|hex",
    "oct|hex",
    "dec|oct|hex",
  };

  for(int i=0; i<7; i++)
    {
      cout << "format : " << str[i] << endl;
      cout.setf(tab[i], ios::basefield);
      fullrange();
    }

  cout.unsetf(ios::dec);
  cout.unsetf(ios::oct);
  cout.unsetf(ios::hex);
  fullrange();

  dec(cout);
}

void testnumbers()
{
  cout << "testnumbers()" << endl;

  cout << "showpos" << endl << showpos;
  fullbase();
  cout << noshowpos;

  cout << "showbase" << endl << showbase;
  fullbase();
  cout << noshowbase;

  cout << "uppercase" << endl << uppercase;
  fullbase();
  cout << nouppercase;


  cout << "showpos" << endl << showpos;
  cout << "showbase" << endl << showbase;
  fullbase();
  cout << noshowpos;
  cout << noshowbase;

  cout << "showpos" << endl << showpos;
  cout << "uppercase" << endl << uppercase;
  fullbase();
  cout << nouppercase;
  cout << noshowpos;

  cout << "showbase" << endl << showbase;
  cout << "uppercase" << endl << uppercase;
  fullbase();
  cout << nouppercase;
  cout << noshowbase;


  cout << "showpos" << endl << showpos;
  cout << "showbase" << endl << showbase;
  cout << "uppercase" << endl << uppercase;
  fullbase();
  cout << nouppercase;
  cout << noshowbase;
  cout << noshowpos;
}

void primetest()
{
  const size_t max=100;
  char *space=new char[max];
  memset(space, 1, max);
  // calculate some primes
  const size_t sq=static_cast<size_t>(sqrt(static_cast<float>(max))+1.0f);
  size_t i=2;
  while(i<=sq)
    {
      for(size_t x=i+i; x<max; x+=i)
	space[x]=0;
      while(space[++i]==0);
    }

  DS_STRING_NAMESPACE::string filename="prime";
  if(suffix.size())
    {
      filename+='.';
      filename+=suffix;
    }

  int outnum=0;
  {
    ofstream os;
    os.open(filename.c_str());
    for(i=0; i<max; i++)
      if(space[i])
	{
	  os << i << endl;
	  outnum++;
	}
  }

  int innum=0;
  char *space2=new char[max];
  memset(space2, 0, max);
  {
    ifstream is(filename.c_str());
    int x;
    while(is>>x)
      if(x<0 || x>(int)max)
	{
	  cout << "error in primetest(): read " << x << endl;
	  exit(1);
	}
      else
	{
	  space2[x]=1;
	  innum++;
	}
  }
  assert(outnum==innum);

  for(i=0; i<max; i++)
    if(space[i]!=space2[i])
      {
	cout << "error in primetest(): mismatch " << i << '(' << (int)space[i] << ',' << (int)space2[i] << ')' << endl;
	exit(1);
      }

  delete [] space2;
  delete [] space;
}

void testss()
{
  int i;
  const int max=3;
  const char* tab[max] = {
    "hallo",
    "",
    "!$$%&/%&/()()=?",
  };

  ostringstream os;
  for(i=0; i<max; i++)
    os << tab[i] << ' ';

  cout << "ostringstream: " << os.str() << endl;

  istringstream is(os.str());
  DS_STRING_NAMESPACE::string s;
  i=0;
  while(is >> s)
    {
      if(s!=tab[i])
	{
	  cerr << "testss() mismatch1: '" << s << "' '" << tab[i] << "'" << endl;
	  exit(1);
	}
      i++;
    }
  assert(i==max);

  ostringstream os2;
  streambuf *sb=cout.rdbuf(os2.rdbuf());
  for(i=0; i<max; i++)
    cout << tab[i] << ' ';

  istringstream is2(os2.str());
  streambuf *sb2=cin.rdbuf(is2.rdbuf());
  i=0;
  while(cin >> s)
    {
      if(s!=tab[i])
	{
	  cerr << "testss() mismatch2: " << s << " " << tab[i] << endl;
	  exit(1);
	}
      i++;
    }

  cout.rdbuf(sb);
  cin.rdbuf(sb2);
}

void outfloat(float f)
{
  cout << "Normal\n";
  cout.precision(2);
  cout << f << endl;
  printf("%f\n", f);
  cout << setprecision(6);
  cout << f << endl;
  printf("%f\n", f);

  ////////////////
  cout << "Showpoint\n";
  cout << showpoint;
  cout.precision(2);
  cout << f << endl;
  cout << setprecision(6);
  cout << f << endl;
  cout << noshowpoint;

  ////////////////
  cout << "fixed\n";
  cout << fixed;
  cout.precision(2);
  cout << f << endl;
  cout << setprecision(6);
  cout << f << endl;
  cout.unsetf(ios::fixed);

  ////////////////
  cout << "scientific\n";
  cout << scientific;
  cout.precision(2);
  cout << f << endl;
  printf("%2.2e\n", f);
  cout << setprecision(6);
  cout << f << endl;
  printf("%6.6e\n", f);
  cout.unsetf(ios::scientific);
}

void testfloat()
{
  cout << showpos << uppercase;
  outfloat(421.0f);
  outfloat(-0.0123456789f);
}

int main(char argc, char **argv)
{
#if 0 //def USESTRING
  teststring();
#endif

#if 0
  streambuf *sb=cout.rdbuf();
  cin.get(*sb, '');
  return 0;
#endif

#if 1
  if(argc>1)
    suffix=argv[1];

  DS_STRING_NAMESPACE::string filename="num.";
  filename+=suffix;

  {
    ofstream os(filename.c_str());
    streambuf *sb=cout.rdbuf(os.rdbuf());
    testnumbers();
    cout.rdbuf(sb);
  }

  testss();
  primetest();

  ifstream is(filename.c_str());
  while(is)
    {
      int c=is.get();
      if(c>=0)
	putchar(c);
    }
#endif

#if 1
  testfloat();
  assert(cout.widen('a')=='a');
  assert(cout.narrow('a', 'z')=='a');
  cout << static_cast<int>(cout.narrow(1000, 'z')) << endl;
#endif

  return 0;
}
